#include "stdafx.h"
#include <GL/gl.h> 
#include <GL/glu.h> 
#include "uuc3d.hpp"

using UuIcsC3d::FrameData;


/* OpenGL code */ 

//#define WIDTH           400 
//#define HEIGHT          300 
 

/* OpenGL globals, defines, and prototypes */ 
GLfloat latitude, longitude, latinc, longinc; 
GLdouble radius; 
 
#define GLOBE    1 

 
void resize( LONG width, LONG height )
{ 
    glViewport( 0, 0, width, height ); 
    GLfloat aspect = (GLfloat) width / height;  
    glMatrixMode( GL_PROJECTION ); 
    glLoadIdentity(); 
    gluPerspective( 45.0, aspect, 100.0, 10000.0 ); 
    glMatrixMode( GL_MODELVIEW ); 
}   


 
GLvoid createObjects() 
{ 
    GLUquadricObj *quadObj; 
 
    glNewList(GLOBE, GL_COMPILE); 
        quadObj = gluNewQuadric (); 
		gluQuadricDrawStyle(quadObj, GLU_FILL); 
		gluQuadricOrientation(quadObj, GLU_OUTSIDE );
        gluSphere(quadObj, 10.0, 16, 16); 
    glEndList(); 

	/*
    glNewList(CONE, GL_COMPILE); 
        quadObj = gluNewQuadric (); 
        gluQuadricDrawStyle (quadObj, GLU_FILL); 
        gluQuadricNormals (quadObj, GLU_SMOOTH); 
        gluCylinder(quadObj, 0.3, 0.0, 0.6, 15, 10); 
    glEndList(); 
 
    glNewList(CYLINDER, GL_COMPILE); 
        glPushMatrix (); 
        glRotatef ((GLfloat)90.0, (GLfloat)1.0, (GLfloat)0.0, (GLfloat)0.0); 
        glTranslatef ((GLfloat)0.0, (GLfloat)0.0, (GLfloat)-1.0); 
        quadObj = gluNewQuadric (); 
        gluQuadricDrawStyle (quadObj, GLU_FILL); 
        gluQuadricNormals (quadObj, GLU_SMOOTH); 
        gluCylinder (quadObj, 0.3, 0.3, 0.6, 12, 2); 
        glPopMatrix (); 
    glEndList(); 
	*/
} 

void initializeGL(LONG width, LONG height) 
{ 
    GLfloat     maxObjectSize; 
    GLdouble    near_plane, far_plane; 
 
	glClearColor(0,0,0,0);
    glClearDepth( 1.0 ); 
 
    glEnable(GL_DEPTH_TEST); 
 

	resize(width, height);
	glLoadIdentity();
 
    near_plane = 3.0; 
    far_plane = 7.0; 
    maxObjectSize = 3.0F; 
    radius = near_plane + maxObjectSize/2.0; 
 
    latitude = 0.0F; 
    longitude = 0.0F; 
    latinc = 6.0F; 
    longinc = 2.5F; 
 
    createObjects(); 
} 

 
void polarView(GLdouble radius, GLdouble twist, GLdouble latitude, 
           GLdouble longitude) 
{ 
    glTranslated(0.0, 0.0, -radius); 
    glRotated(-twist, 0.0, 0.0, 1.0); 
    glRotated(-latitude, 1.0, 0.0, 0.0); 
    glRotated(longitude, 0.0, 0.0, 1.0);      
 
} 
 
GLvoid drawScene(HDC const &ghDC, int camera, FrameData const *frame_data) 
{ 
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT ); 
	glPushMatrix(); 
	glLoadIdentity();
	switch (camera) {
		case 0:
		default:
			gluLookAt(-4000.0, -4000.0, 800.0, 0.0, 0.0, 800.0, 0.0, 0.0, 1.0);
			break;
		case 1:
			gluLookAt(0.0, -5656.0, 800.0, 0.0, 0.0, 800.0, 0.0, 0.0, 1.0);
			break;
		case 2:
			gluLookAt(4000.0, -4000.0, 800.0, 0.0, 0.0, 800.0, 0.0, 0.0, 1.0);
			break;
		case 3:
			gluLookAt(5656.0, 0.0, 800.0, 0.0, 0.0, 800.0, 0.0, 0.0, 1.0);
			break;
		case 4:
			gluLookAt(4000.0, 4000.0, 800.0, 0.0, 0.0, 800.0, 0.0, 0.0, 1.0);
			break;
		case 5:
			gluLookAt(0.0, 5656.0, 800.0, 0.0, 0.0, 800.0, 0.0, 0.0, 1.0);
			break;
		case 6:
			gluLookAt(-4000.0, 4000.0, 800.0, 0.0, 0.0, 800.0, 0.0, 0.0, 1.0);
			break;
		case 7:
			gluLookAt(-5656.0, 0.0, 800.0, 0.0, 0.0, 800.0, 0.0, 0.0, 1.0);
			break;
	}



	glColor3d(1.0,1.0,0.0);
	glPushMatrix();
	glTranslated(0., 500.0,0.0);
	glCallList(GLOBE);
	glPopMatrix();
	glPushMatrix();
	glTranslated(-50.0,-500.0,0.0);
	glCallList(GLOBE);
	glPopMatrix();
	glColor3d(0.0, 0.99, 0.99);
	if (frame_data) {
		//float min[] = {1e20f, 1e20f, 1e20f};
		//float max[] = {-1e20f, -1e20f, -1e20f};
		for (size_t i=0; i<frame_data->points.size(); ++i) {
			if (frame_data->points[i].is_valid()) {
				UuIcsC3d::DataPoint3d::CTripleF coords =frame_data->points[i].coordinates();
				/*
				for (int j=0; j<3; ++j) {
					if (min[j]>coords[j])
						min[j]=coords[j];
					if (max[j]<coords[j])
						max[j]=coords[j];
				}
				*/
				glPushMatrix(); 
				glTranslatef(coords[0],coords[1],coords[2]);
				glCallList(GLOBE);
				glPopMatrix(); 
			}
		}
		//int dummy = 12;
	}
	glPopMatrix(); 
	SwapBuffers(ghDC); 
}
